/***************************************************************************
                          smb4kglobal  -  description
                             -------------------
    begin                : Sa Apr 2 2005
    copyright            : (C) 2005 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>

#include <kconfig.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>

// application specific includes
#include "smb4kglobal.h"


/****************************************************************************
   Returns the global options for smbclient
****************************************************************************/

const QString Smb4K_Global::getSmbclientOptions()
{
  KConfig *config = new KConfig( "smb4krc", false, false );

  QString options;

  config->setGroup( "Samba" );

  if ( !config->readEntry( "Client Resolve Order", QString::null ).isEmpty() )
  {
    options.append( QString( " -R %1" ).arg( KProcess::quote( config->readEntry( "Client Resolve Order", QString::null ) ) ) );
  }

  options.append( QString( " -p %1" ).arg( config->readNumEntry( "Port", 139 ) ) );

  if ( !config->readEntry( "Client Buffer Size", QString::null ).isEmpty() )
  {
    options.append( QString( " -b %1" ).arg( config->readNumEntry( "Client Buffer Size", 65520 ) ) );
  }

  if ( config->readBoolEntry( "Use Kerberos", false ) )
  {
    if ( config->readEntry( "Version", QString::null ).startsWith( "3" ) )
    {
      options.append( " -k" );
    }
  }

  if ( !config->readEntry( "NetBIOS Name", QString::null ).isEmpty() )
  {
    options.append( QString( " -n %1" ).arg( KProcess::quote( config->readEntry( "NetBIOS Name", QString::null ) ) ) );
  }

  if ( !config->readEntry( "NetBIOS Scope", QString::null ).isEmpty() )
  {
    options.append( QString( " -i %1" ).arg( KProcess::quote( config->readEntry( "NetBIOS Scope", QString::null ) ) ) );
  }

  if ( !config->readEntry( "Socket Options", QString::null ).isEmpty() )
  {
    options.append( QString( " -O %1" ).arg( KProcess::quote( config->readEntry( "Socket Options", QString::null ) ) ) );
  }

  delete config;

  return options;
}


/****************************************************************************
   Returns the global options for nmblookup
****************************************************************************/

const QString Smb4K_Global::getNmblookupOptions()
{
  KConfig *config = new KConfig( "smb4krc", false, false );

  QString options;

  config->setGroup( "Samba" );

  if ( !config->readEntry( "NetBIOS Name", QString::null ).isEmpty() )
  {
    options.append( QString( " -n %1" ).arg( KProcess::quote( config->readEntry( "NetBIOS Name", QString::null ) ) ) );
  }

  if ( !config->readEntry( "NetBIOS Scope", QString::null ).isEmpty() )
  {
    options.append( QString( " -i %1" ).arg( KProcess::quote( config->readEntry( "NetBIOS Scope", QString::null ) ) ) );
  }

  if ( !config->readEntry( "Socket Options", QString::null ).isEmpty() )
  {
    options.append( QString( " -O %1" ).arg( KProcess::quote( config->readEntry( "Socket Options", QString::null ) ) ) );
  }

  if ( !config->readEntry( "Domain", QString::null ).isEmpty() )
  {
    if ( config->readEntry( "Version", QString::null ).startsWith( "3" ) )
    {
      options.append( QString( " -W %1" ).arg( KProcess::quote( config->readEntry( "Domain", QString::null ) ) ) );
    }
  }

  if ( !config->readEntry( "NMB Broadcast", QString::null ).isEmpty() )
  {
    options.append( QString( " -B %1" ).arg( KProcess::quote( config->readEntry( "NMB Broadcast", QString::null ) ) ) );
  }

  if ( config->readBoolEntry( "NMB Port 137", false ) )
  {
    options.append( " -r" );
  }

  delete config;

  return options;
}


/****************************************************************************
   Reads the [global section of the smb.conf file
****************************************************************************/

const QMap<QString,QString> Smb4K_Global::readGlobalSMBOptions()
{
  QStringList paths;
  paths << "/etc";
  paths << "/etc/samba";
  paths << "/usr/local/etc";
  paths << "/usr/local/etc/samba";

  QFile f( "smb.conf" );

  QStringList contents;

  for ( QStringList::Iterator it = paths.begin(); it != paths.end(); ++it )
  {
    QDir::setCurrent( *it );

    if ( f.exists() )
    {
      if ( f.open( IO_ReadOnly ) )
      {
        QTextStream ts( &f );

        contents = QStringList::split( '\n', ts.read(), false );
      }

      f.close();

      break;
    }
    else
    {
      continue;
    }
  }

  for ( QStringList::Iterator it = contents.erase( contents.begin(), ++(contents.find( "[global]" )) ); it != contents.end(); ++it )
  {
    if ( (*it).startsWith( "#" ) )
    {
      *it = QString::null;
    }
    else if ( (*it).startsWith( "[" ) )
    {
      contents.erase( it, contents.end() );
      break;
    }
  }

  contents.remove( QString::null );

  QMap<QString,QString> map;

  // Write all options into the map:
  for ( QStringList::ConstIterator it = contents.begin(); it != contents.end(); ++it )
  {
    QString key = (*it).section( "=", 0, 0 ).stripWhiteSpace().lower();
    map[key] = QString( (*it).section( "=", 1, 1 ).stripWhiteSpace().upper() );
  }

  // Post-processing. Some values should be entered with their defaults, if they are
  // not already present.
  if ( !map.contains( "netbios name" ) )
  {
    char *hostname;
    size_t hostnamelen = 255;
    hostname = (char *)malloc( hostnamelen );

    if ( gethostname( hostname, hostnamelen ) == 0 )
    {
      map["netbios name"] = ( QString( "%1" ).arg( hostname ) ).upper();
    }

    free(hostname);
  }

  return map;
}



/****************************************************************************
   Returns the WINS server the system is using
****************************************************************************/

const QString Smb4K_Global::getWINSServer()
{
  QString wins = QString::null;
  QMap<QString, QString> options = readGlobalSMBOptions();

  if ( options.find( "wins server" ) != options.end() )
  {
    wins = options["wins server"];
  }
  else if ( options.find( "wins support" ) != options.end() && 
            (QString::compare( options["wins support"].lower(), "yes" ) == 0 || 
             QString::compare( options["wins support"].lower(), "true" ) == 0) )
  {
    wins = "127.0.0.1";
  }

  return wins;
}


/****************************************************************************
   Returns the kernel version
****************************************************************************/

const QString Smb4K_Global::getKernelVersion()
{
  struct utsname system_info;

  uname( &system_info );

  return QString( "%1" ).arg( system_info.release ).section( "-", 0, 0 );
}


/****************************************************************************
   Returns the system name
****************************************************************************/

const QString Smb4K_Global::getSystemName()
{
  struct utsname system_info;

  uname( &system_info );

  return QString( "%1" ).arg( system_info.sysname );
}


/****************************************************************************
   Returns the UMASK
****************************************************************************/

int Smb4K_Global::getUMASK()
{
  return umask( 0 );
}


/****************************************************************************
   Returns the global options of the net command
****************************************************************************/

const QString Smb4K_Global::getNetOptions( const QString &command )
{
  KConfig *config = new KConfig( "smb4krc", false, false );
  QMap<QString,QString> smb_map = readGlobalSMBOptions();

  QString options;

  if ( QString::compare( command, "share" ) == 0 )
  {
    config->setGroup( "Samba" );

    if ( QString::compare( config->readEntry( "Net Protocol", "auto" ), "auto" ) != 0 )
    {
      options.append( " "+config->readEntry( "Net Protocol", "auto" ) );
    }

    options.append( " share" );

    if ( config->readBoolEntry( "Net Machine Account", false ) )
    {
      options.append( " -P" );
    }

    options.append( QString( " -p %1" ).arg( config->readNumEntry( "Port", 139 ) ) );
    options.append( " -l" );
    options.append( QString( " -W %1" ).arg( KProcess::quote( config->readEntry( "Domain", smb_map["workgroup"] ) ) ) );
    options.append( QString( " -n %1" ).arg( KProcess::quote( config->readEntry( "NetBIOS Name", smb_map["netbios name"] ) ) ) );
  }
  else
  {
    KMessageBox::error( 0, i18n( "The command '%1' is not supported by Smb4K." ).arg( command ) );
  }

  delete config;

  return options;
}

